import { TensorScriptOptions, TensorScriptProperties, Matrix, PredictionOptions, Vector } from './model_interface';
import { MachineLearningModelInterface } from './ml_model_interface';
/**
 * Machine Learning Linear Regression with Tensorflow
 * @class MachineLearningLinearRegression
 * @implements {MachineLearningModelInterface}
 */
export declare class MachineLearningLinearRegression extends MachineLearningModelInterface {
    /**
     * @param {{layers:Array<Object>,compile:Object,fit:Object,layerPreference:String}} options - neural network configuration and tensorflow model hyperparameters
     * @param {{model:Object,tf:Object,}} properties - extra instance properties
     */
    constructor(options?: TensorScriptOptions, properties?: TensorScriptProperties);
    /**
     * Asynchronously trains tensorflow model
     * @override
     * @param {Array<Array<number>>} x_matrix - independent variables
     * @param {Array<Array<number>>} y_matrix - dependent variables
     * @return {Object} returns trained tensorflow model
     */
    train(x_matrix: Matrix, y_matrix: Matrix): Promise<any>;
    /**
     * Predicts new dependent variables
     * @override
     * @param {Array<Array<number>>|Array<number>} matrix - new test independent variables
     * @param {Object} options - model prediction options
     * @return {{data: Promise}} returns tensorflow prediction
     */
    calculate(input_matrix: Matrix | Vector, options?: PredictionOptions): any;
}
